/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.util.Properties;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.EBSInterface;
import edu.umass.cs.geni.dicloud.server.EBSServer;

/**
 * Probe an EBS volume
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class EBSProbe extends ConsoleCommand
{

  /**
   * @param module
   */
  public EBSProbe(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Probe an EBS volume and get current status information";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "ebs probe";
  }

  @Override
  public String getCommandParameters()
  {
	return "cert.pem pk.pem volume_id ec2_region ascii|html";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);

	if (st.countTokens() != 5)
	{
	  console.printString("Invalid number of parameters for this command");
	  return;
	}

	// Probe EBS volume
	Properties p = new Properties();
	p.put(AmazonProperties.AWS_CERT_FILE, st.nextToken());
	p.put(AmazonProperties.AWS_PRIVATE_KEY_FILE, st.nextToken());
	p.put(AmazonProperties.EBS_VOLUME_ID, st.nextToken());
	p.put(AmazonProperties.EC2_REGION, st.nextToken());
	String lineReturn;
	if ("html".equalsIgnoreCase(st.nextToken()))
	  lineReturn = "<br/>\n";
	else
	  lineReturn = "\n";

	EBSInterface ebsServer = (EBSInterface) Naming.lookup(module.getServerUrl()
	    + "/" + EBSServer.RMI_NAME_REGISTRATION);
	try
	{
	  Properties props = ebsServer.probe(p);
	  console.printString("EBS Volume id: "
		  + props.getProperty(AmazonProperties.EBS_VOLUME_ID) + lineReturn);
	  console.printString("Region: "
		  + props.getProperty(AmazonProperties.EC2_REGION) + lineReturn);
	  console.printString("Creation time: "
		  + props.getProperty(AmazonProperties.EBS_CREATION_TIME) + lineReturn);
	  console.printString("Status: "
		  + props.getProperty(AmazonProperties.EBS_STATUS) + lineReturn);
	  String instanceId = props.getProperty(AmazonProperties.EC2_INSTANCE_ID);
	  if ((instanceId != null) && (!"".equals(instanceId)))
	  {
		console.printString("Attached to EC2 instance id: "
		    + props.getProperty(AmazonProperties.EC2_INSTANCE_ID) + lineReturn);
		console.printString("Registered as device : "
		    + props.getProperty(AmazonProperties.EBS_DEVICE) + lineReturn);
		console.printString("Attach time: "
		    + props.getProperty(AmazonProperties.EBS_ATTACH_TIME) + lineReturn);

	  }
	}
	catch (Exception e)
	{
	  console.printString("EBS probe failed: " + e.getMessage());
	}
  }
}
