/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.util.Properties;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.EC2Interface;
import edu.umass.cs.geni.dicloud.server.EC2Server;

/**
 * Probe an EC2 instance to get all details about it
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class EC2ProbeInstance extends ConsoleCommand
{

  /**
   * @param module
   */
  public EC2ProbeInstance(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Check the status of a running EC2 instance";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "ec2 probe";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandParameters
   * ()
   */
  @Override
  public String getCommandParameters()
  {
	return "cert.pem pk.pem instance_id ec2_region html|ascii";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);

	if (st.countTokens() != 5)
	{
	  console.printString("Invalid number of parameters for this command");
	  return;
	}

	// Probe an EC2 instance
	Properties p = new Properties();
	p.put(AmazonProperties.AWS_CERT_FILE, st.nextToken());
	p.put(AmazonProperties.AWS_PRIVATE_KEY_FILE, st.nextToken());
	p.put(AmazonProperties.EC2_INSTANCE_ID, st.nextToken());
	p.put(AmazonProperties.EC2_REGION, st.nextToken());

	// Launch the instance
	EC2Interface ec2Server = (EC2Interface) Naming.lookup(module.getServerUrl()
	    + "/" + EC2Server.RMI_NAME_REGISTRATION);
	try
	{
	  Properties result = ec2Server.describeInstance(p);

	  if ("html".equalsIgnoreCase(st.nextToken()))
	  { // HTML mode
		console.printString("<table border=\"1\">");
		console.printString("<tr><td>Instance Id</td><td>"
		    + result.get(AmazonProperties.EC2_INSTANCE_ID) + "</td></tr>");
		console.printString("<tr><td>AMI Id</td><td>"
		    + result.get(AmazonProperties.EC2_AMI_ID) + "</td></tr>");
		console.printString("<tr><td>Public name</td><td>"
		    + result.get(AmazonProperties.EC2_PUBLIC_NAME) + "</td></tr>");
		console.printString("<tr><td>Private name</td><td>"
		    + result.get(AmazonProperties.EC2_PRIVATE_NAME) + "</td></tr>");
		console.printString("<tr><td>Public IP</td><td>"
		    + result.get(AmazonProperties.EC2_PUBLIC_IP) + "</td></tr>");
		console.printString("<tr><td>Private IP</td><td>"
		    + result.get(AmazonProperties.EC2_PRIVATE_IP) + "</td></tr>");
		console.printString("<tr><td>Instance State</td><td>"
		    + result.get(AmazonProperties.EC2_INSTANCE_STATE) + "</td></tr>");
		console.printString("<tr><td>Instance Type</td><td>"
		    + result.get(AmazonProperties.EC2_INSTANCE_TYPE) + "</td></tr>");
		console.printString("<tr><td>Key pair</td><td>"
		    + result.get(AmazonProperties.EC2_KEY_PAIR_NAME) + "</td></tr>");
		console.printString("<tr><td>Start time</td><td>"
		    + result.get(AmazonProperties.EC2_START_TIME) + "</td></tr>");
		console
		    .printString("<tr><td>Availability zone</td><td>"
		        + result.get(AmazonProperties.EC2_AVAILABILITY_ZONE)
		        + "</td></tr>");
		console.printString("</table>");
	  }
	  else
	  { // ASCII output
		console.printString("Instance Id: "
		    + result.get(AmazonProperties.EC2_INSTANCE_ID) + "\n");
		console.printString("AMI Id: "
		    + result.get(AmazonProperties.EC2_AMI_ID) + "\n");
		console.printString("Public name: "
		    + result.get(AmazonProperties.EC2_PUBLIC_NAME) + "\n");
		console.printString("Private name: "
		    + result.get(AmazonProperties.EC2_PRIVATE_NAME) + "\n");
		console.printString("Public IP: "
		    + result.get(AmazonProperties.EC2_PUBLIC_IP) + "\n");
		console.printString("Private IP: "
		    + result.get(AmazonProperties.EC2_PRIVATE_IP) + "\n");
		console.printString("Instance State: "
		    + result.get(AmazonProperties.EC2_INSTANCE_STATE) + "\n");
		console.printString("Instance Type: "
		    + result.get(AmazonProperties.EC2_INSTANCE_TYPE) + "\n");
		console.printString("Key pair: "
		    + result.get(AmazonProperties.EC2_KEY_PAIR_NAME) + "\n");
		console.printString("Start time: "
		    + result.get(AmazonProperties.EC2_START_TIME) + "\n");
		console.printString("Availability zone: "
		    + result.get(AmazonProperties.EC2_AVAILABILITY_ZONE) + "\n");
	  }
	}
	catch (Exception e)
	{
	  console.printString("Probing failed: " + e.getMessage());
	}
  }
}
