/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.util.Properties;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.S3Interface;
import edu.umass.cs.geni.dicloud.server.S3Server;

/**
 * Create an S3 bucket
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class S3Create extends ConsoleCommand
{

  /**
   * @param module
   */
  public S3Create(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Create an S3 bucket";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "s3 create";
  }

  @Override
  public String getCommandParameters()
  {
	return "bucket_name region";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);

	if (st.countTokens() != 2)
	{
	  console.printString("Invalid number of parameters for this command");
	  return;
	}

	// Create S3 bucket
	Properties p = new Properties();
	p.put(AmazonProperties.S3_BUCKET_NAME, st.nextToken());
	p.put(AmazonProperties.S3_LOCATION, st.nextToken());

	S3Interface s3Server = (S3Interface) Naming.lookup(module.getServerUrl()
	    + "/" + S3Server.RMI_NAME_REGISTRATION);
	try
	{
	  s3Server.create(p);
	}
	catch (Exception e)
	{
	  console.printString("S3 create failed: " + e.getMessage());
	}
  }

}
