/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.io.File;
import java.io.FileOutputStream;
import java.rmi.Naming;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.S3ProxyInterface;

/**
 * Get an object from an S3 bucket
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class S3Get extends ConsoleCommand
{

  /**
   * @param module
   */
  public S3Get(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Get the content of an object named object_name into the specified file from the S3 bucket";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "s3 get";
  }

  @Override
  public String getCommandParameters()
  {
	return "s3_bucket_name object_name file_path";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);

	if (st.countTokens() != 3)
	{
	  console.printString("Invalid number of parameters for this command");
	  return;
	}

	byte[] data = null;
	String bucketName = st.nextToken();
	String objectName = st.nextToken();
	String fileName = st.nextToken();

	S3ProxyInterface s3Proxy = (S3ProxyInterface) Naming.lookup(module
	    .getServerUrl()
	    + "/" + bucketName);

	console.printString("Downloading file " + fileName + " from object "
	    + objectName + " in bucket " + bucketName);
	try
	{
	  data = s3Proxy.get(objectName);
	}
	catch (Exception e)
	{
	  console.printString("Failed to get object: " + e.getMessage());
	}
	try
	{
	  File f = new File(fileName);
	  f.createNewFile();
	  // if a file with that name already exists, it wil fail here
	  FileOutputStream fos = new FileOutputStream(f);
	  fos.write(data);
	  fos.close();
	}
	catch (Exception e)
	{
	  console.printString("Failed to create file: " + e.getMessage());
	}
  }

}
