/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.server;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.List;

import edu.umass.cs.geni.dicloud.accounting.AWSActivity;

/**
 * Interface to the AWS accounting services offered by the server
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public interface AWSAccoutingInterface extends Remote
{

  /**
   * Add credit to the AWS account
   * 
   * @param credit
   *          the credit to add
   * @throws RemoteException
   *           if an RMI error occurs
   * @throws Exception
   *           if another error occurs
   */
  public abstract void addCredit(float credit) throws RemoteException,
	  Exception;

  /**
   * Retrieve the current balance of the account
   * 
   * @return the balance (positive if credit, negative if debit)
   * @throws Exception
   *           if an error occurs accessing the database
   */
  public abstract double getBalance() throws RemoteException, Exception;

  /**
   * Retrieve the history of activities on the AWS account
   * 
   * @param historySize
   *          How many records to fetch from the database (0 means not limit)
   * @return List of activities
   * @throws Exception
   *           if an error occurs accessing the database
   */
  public abstract List<AWSActivity> getHistory(int historySize)
	  throws RemoteException, Exception;

}
