/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.server;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Properties;

/**
 * TODO: Def here
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public interface EBSInterface extends Remote
{
  /**
   * Create a new EBS volume by invoking ec2-create-volume.
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.AWS_PRIVATE_KEY_FILE: AWS account private key
   *  - AmazonProperties.AWS_CERT_FILE: AWS account X509 certificate
   *  - AmazonProperties.EBS_SIZE: AMI id to use to create instances
   *  - AmazonProperties.EC2_AVAILABILITY_ZONE: availability zone where to create the instance
   * Optional properties: 
   *  - AmazonProperties.EC2_REGION: region where to create the instance
   * </pre>
   * 
   * Returns results in the following properties:
   * 
   * <pre>
   * Config.PropertyTargetResultCode: ResultCodeOK on success, ResultCodeException on error
   * Config.PropertyTargetResultCodeMessage: command output (if success)
   * Config.PropertyExceptionMessage: Error message (if error)
   * Config.PropertyExceptionStack: stderr content if command failed
   * 
   * AmazonProperties.EBS_VOLUME_ID: Id of the EBS volume
   * AmazonProperties.EBS_CREATION_TIME: Creation time of the volume
   * AmazonProperties.EC2_REGION: region where the volume has been created
   * AmazonProperties.EBS_STATUS: status of the volume
   * AmazonProperties.EBS_CREATION_TIME: volume creation time
   * </pre>
   */
  public Properties create(Properties p) throws Exception, RemoteException;

  /**
   * Delete an EBS volume by invoking ec2-delete-volume
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.AWS_PRIVATE_KEY_FILE: AWS account private key
   *  - AmazonProperties.AWS_CERT_FILE: AWS account X509 certificate
   *  - AmazonProperties.EBS_VOLUME_ID: EBS volume id to delete
   * Optional properties: 
   *  - AmazonProperties.EC2_REGION: region where to create the instance
   * </pre>
   * 
   * Returns results in the following properties:
   * 
   * <pre>
   * Config.PropertyTargetResultCode: ResultCodeOK on success, ResultCodeException on error
   * Config.PropertyTargetResultCodeMessage: command output (if success)
   * Config.PropertyExceptionMessage: Error message (if error)
   * Config.PropertyExceptionStack: stderr content if command failed
   * </pre>
   */
  public Properties delete(Properties p) throws Exception, RemoteException;

  /**
   * Invoke ec2-describe-volumes to probe the EBS volume. The expected
   * properties are:
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.EC2_AWS_PRIVATE_KEY_FILE: AWS account private key
   *  - AmazonProperties.EC2_AWS_CERT_FILE: AWS account X509 certificate
   *  - AmazonProperties.EBS_VOLUME_ID: EBS volume id to probe
   * Optional properties: 
   *  - AmazonProperties.EC2_REGION: region where to create the instance
   * </pre>
   * 
   * Returns results in the following properties:
   * 
   * <pre>
   * Config.PropertyTargetResultCode: ResultCodeOK on success, ResultCodeException on error
   * Config.PropertyTargetResultCodeMessage: AMI status (if success)
   * Config.PropertyExceptionMessage: Error message (if error)
   * Config.PropertyExceptionStack: stderr content if command failed
   * 
   * AmazonProperties.EBS_VOLUME_ID: Id of the EBS volume
   * AmazonProperties.EBS_CREATION_TIME: Creation time of the volume
   * AmazonProperties.REGION: region where the volume has been created
   * AmazonProperties.EBS_STATUS: status of the volume
   * AmazonProperties.EBS_CREATION_TIME: volume creation time
   * </pre>
   */
  public Properties probe(Properties p) throws Exception, RemoteException;

  /**
   * Invoke ec2-attach-volume to attach an EBS volume to a running instance. The
   * expected properties are:
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.EC2_AWS_PRIVATE_KEY_FILE: AWS account private key
   *  - AmazonProperties.EC2_AWS_CERT_FILE: AWS account X509 certificate
   *  - AmazonProperties.EBS_VOLUME_ID: EBS volume id to attach
   *  - AmazonProperties.EC2_INSTANCE_ID: Instance to attach to
   *  - AmazonProperties.EBS_DEVICE:  Device name as which to expose VOLUME inside INSTANCE.
   * Optional properties: 
   *  - AmazonProperties.EC2_REGION: region where to create the instance
   * </pre>
   * 
   * Returns results in the following properties:
   * 
   * <pre>
   * Config.PropertyTargetResultCode: ResultCodeOK on success, ResultCodeException on error
   * Config.PropertyTargetResultCodeMessage: AMI status (if success)
   * Config.PropertyExceptionMessage: Error message (if error)
   * Config.PropertyExceptionStack: stderr content if command failed
   * 
   * AmazonProperties.EBS_STATUS: status of the volume
   * </pre>
   */
  public Properties attach(Properties p) throws Exception, RemoteException;

  /**
   * Invoke ec2-detach-volume to attach an EBS volume to a running instance. The
   * expected properties are:
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.EC2_AWS_PRIVATE_KEY_FILE: AWS account private key
   *  - AmazonProperties.EC2_AWS_CERT_FILE: AWS account X509 certificate
   *  - AmazonProperties.EBS_VOLUME_ID: EBS volume id to attach
   *  - AmazonProperties.EC2_INSTANCE_ID: Instance to attach to
   *  - AmazonProperties.EBS_DEVICE:  Device name as which to expose VOLUME inside INSTANCE.
   * Optional properties: 
   *  - AmazonProperties.EC2_REGION: region where to create the instance
   * </pre>
   * 
   * Returns results in the following properties:
   * 
   * <pre>
   * Config.PropertyTargetResultCode: ResultCodeOK on success, ResultCodeException on error
   * Config.PropertyTargetResultCodeMessage: AMI status (if success)
   * Config.PropertyExceptionMessage: Error message (if error)
   * Config.PropertyExceptionStack: stderr content if command failed
   * 
   * AmazonProperties.EBS_STATUS: status of the volume
   * </pre>
   */
  public Properties detach(Properties p) throws Exception, RemoteException;

}
