/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Properties;

import orca.shirako.plugins.ShirakoPlugin;
import orca.shirako.plugins.config.Config;

import org.apache.log4j.Logger;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.handlers.EBSHandler;

/**
 * TODO: Def here
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class EBSServer extends UnicastRemoteObject implements EBSInterface
{
  private static final long  serialVersionUID      = -8956186791111264180L;
  private EBSHandler         handler;
  private ShirakoPlugin      plugin;
  private Logger             logger                = Logger
	                                                   .getLogger(EC2Server.class);

  public static final String RMI_NAME_REGISTRATION = "dicloud/ebs";

  /**
   * @throws RemoteException
   */
  public EBSServer(ShirakoPlugin plugin, EBSHandler handler)
	  throws RemoteException
  {
	this.plugin = plugin;
	this.handler = handler;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.server.EBSInterface#create(java.util.Properties)
   */
  public Properties create(Properties p) throws Exception, RemoteException
  {
	handler.join(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	String status = null;

	if ("0".equals(result))
	{
	  status = props.getProperty(Config.PropertyTargetResultCodeMessage);
	  logger.debug("Command status: " + status);
	  logger.debug("Volume id: "
		  + props.getProperty(AmazonProperties.EBS_VOLUME_ID));
	  logger.debug("Creation time: "
		  + props.getProperty(AmazonProperties.EBS_CREATION_TIME));
	  logger.debug("EBS Status: "
		  + props.getProperty(AmazonProperties.EBS_STATUS));
	  logger.debug("Region: " + props.getProperty(AmazonProperties.EC2_REGION));
	}
	else
	{
	  logger.debug("EBS create result code: " + result);
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to probe EBS volume ("
		  + props.getProperty(Config.PropertyExceptionMessage) + " because of "
		  + Config.PropertyExceptionStack + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.server.EBSInterface#delete(java.util.Properties)
   */
  public Properties delete(Properties p) throws Exception, RemoteException
  {
	handler.leave(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if (!"0".equals(result))
	{
	  logger.debug("EBS delete result code: " + result);
	  logger.debug("Status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to delete EBS volume ("
		  + props.getProperty(Config.PropertyExceptionMessage) + " because of "
		  + Config.PropertyExceptionStack + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.server.EBSInterface#probe(java.util.Properties)
   */
  public Properties probe(Properties p) throws Exception, RemoteException
  {
	handler.probe(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	String status = null;

	if ("0".equals(result))
	{
	  status = props.getProperty(Config.PropertyTargetResultCodeMessage);
	  logger.debug("Command status: " + status);
	  logger.debug("Volume id: "
		  + props.getProperty(AmazonProperties.EBS_VOLUME_ID));
	  logger.debug("Creation time: "
		  + props.getProperty(AmazonProperties.EBS_CREATION_TIME));
	  logger.debug("EBS Status: "
		  + props.getProperty(AmazonProperties.EBS_STATUS));
	  logger.debug("Region: " + props.getProperty(AmazonProperties.EC2_REGION));
	}
	else
	{
	  logger.debug("EBS Probing result code: " + result);
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to probe EBS volume ("
		  + props.getProperty(Config.PropertyExceptionMessage) + " because of "
		  + Config.PropertyExceptionStack + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.server.EBSInterface#attach(java.util.Properties)
   */
  public Properties attach(Properties p) throws Exception, RemoteException
  {
	handler.attach(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	String status = null;

	if ("0".equals(result))
	{
	  status = props.getProperty(Config.PropertyTargetResultCodeMessage);
	  logger.debug("Command status: " + status);
	  logger.debug("EBS Status: "
		  + props.getProperty(AmazonProperties.EBS_STATUS));
	}
	else
	{
	  logger.debug("EBS attach result code: " + result);
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to attach EBS volume ("
		  + props.getProperty(Config.PropertyExceptionMessage) + " because of "
		  + Config.PropertyExceptionStack + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.server.EBSInterface#detach(java.util.Properties)
   */
  public Properties detach(Properties p) throws Exception, RemoteException
  {
	handler.detach(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	String status = null;

	if ("0".equals(result))
	{
	  status = props.getProperty(Config.PropertyTargetResultCodeMessage);
	  logger.debug("Command status: " + status);
	  logger.debug("EBS Status: "
		  + props.getProperty(AmazonProperties.EBS_STATUS));
	}
	else
	{
	  logger.debug("EBS detach result code: " + result);
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to detach EBS volume ("
		  + props.getProperty(Config.PropertyExceptionMessage) + " because of "
		  + Config.PropertyExceptionStack + ")");
	}
	return props;
  }

}
