/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Properties;

import orca.shirako.plugins.ShirakoPlugin;
import orca.shirako.plugins.config.Config;

import org.apache.log4j.Logger;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.handlers.EC2Handler;

/**
 * TODO: Def here
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class EC2Server extends UnicastRemoteObject implements EC2Interface
{
  private static final long  serialVersionUID      = 2142294012470342202L;
  private EC2Handler         handler;
  private ShirakoPlugin      plugin;
  private Logger             logger                = Logger
	                                                   .getLogger(EC2Server.class);

  public static final String RMI_NAME_REGISTRATION = "dicloud/ec2";

  /**
   * @throws RemoteException
   */
  public EC2Server(ShirakoPlugin plugin, EC2Handler handler)
	  throws RemoteException
  {
	this.plugin = plugin;
	this.handler = handler;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.server.EC2Interface#describeInstance(java.util
   * .Properties)
   */
  public Properties describeInstance(Properties p) throws Exception,
	  RemoteException
  {
	handler.probe(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	String status = null;

	if ("0".equals(result))
	{
	  status = props.getProperty(Config.PropertyTargetResultCodeMessage);
	  logger.debug("AMI status: " + status);
	  logger.debug("Instance id: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_ID));
	  logger.debug("AMI id: " + props.getProperty(AmazonProperties.EC2_AMI_ID));
	  logger.debug("Public name: "
		  + props.getProperty(AmazonProperties.EC2_PUBLIC_NAME));
	  logger.debug("Private name: "
		  + props.getProperty(AmazonProperties.EC2_PRIVATE_NAME));
	  logger.debug("Public IP: "
		  + props.getProperty(AmazonProperties.EC2_PUBLIC_IP));
	  logger.debug("Private IP: "
		  + props.getProperty(AmazonProperties.EC2_PRIVATE_IP));
	  logger.debug("Instance state: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_STATE));
	  logger.debug("Instance type: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_TYPE));
	  logger.debug("Key pair: "
		  + props.getProperty(AmazonProperties.EC2_KEY_PAIR_NAME));
	  logger.debug("Start time: "
		  + props.getProperty(AmazonProperties.EC2_START_TIME));
	  logger.debug("Availability zone: "
		  + props.getProperty(AmazonProperties.EC2_AVAILABILITY_ZONE));
	}
	else
	{
	  logger.debug("EC2 Probing result code: " + result);
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to probe EC2 instance ("
		  + props.getProperty(Config.PropertyExceptionMessage) + "because of "
		  + props.getProperty(Config.PropertyExceptionStack) + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @seeedu.umass.cs.geni.dicloud.server.EC2Interface#startInstance(java.util.
   * Properties)
   */
  public Properties startInstance(Properties p) throws Exception,
	  RemoteException
  {
	handler.join(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if ("0".equals(result))
	{
	  logger.debug("Key pair: "
		  + props.getProperty(AmazonProperties.EC2_KEY_PAIR_NAME) + ".pem\n"
		  + props.getProperty(AmazonProperties.EC2_KEY_PAIR) + "\n");
	  logger.debug("AMI status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	}
	else
	{
	  logger.debug("EC2 Join result code: " + result);
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to create EC2 instance ("
		  + props.getProperty(Config.PropertyExceptionMessage) + "because of "
		  + props.getProperty(Config.PropertyExceptionStack) + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.server.EC2Interface#stopInstance(java.util.Properties
   * )
   */
  public Properties stopInstance(Properties p) throws Exception,
	  RemoteException
  {
	handler.leave(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if (!"0".equals(result))
	{
	  logger.debug("EC2 Leave result code: " + result);
	  logger.debug("Status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to terminate EC2 instance ("
		  + props.getProperty(Config.PropertyExceptionMessage) + "because of "
		  + props.getProperty(Config.PropertyExceptionStack) + ")");
	}
	return props;
  }

}
