/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.server;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Properties;

/**
 * TODO: Def here
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public interface S3Interface extends Remote
{

  /**
   * Create a new S3 bucket invoking 's3cmd mb'.
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.S3_BUCKET_NAME: name of the S3 bucket to create
   * Optional properties: 
   *  - AmazonProperties.S3_LOCATION: region where to create the bucket
   * </pre>
   * 
   * Returns results in the following properties:
   * 
   * <pre>
   * Config.PropertyTargetResultCode: ResultCodeOK on success, ResultCodeException on error
   * Config.PropertyTargetResultCodeMessage: command output (if success)
   * Config.PropertyExceptionMessage: Error message (if error)
   * Config.PropertyExceptionStack: stderr content if command failed
   * </pre>
   */
  public Properties create(Properties p) throws Exception, RemoteException;

  /**
   * Delete an S3 bucket by invoking 's3cmd rb'
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.S3_BUCKET_NAME: name of the S3 bucket to create
   * Optional properties: 
   *  - AmazonProperties.S3_LOCATION: region where to create the bucket
   * </pre>
   * 
   * Returns results in the following properties:
   * 
   * <pre>
   * Config.PropertyTargetResultCode: ResultCodeOK on success, ResultCodeException on error
   * Config.PropertyTargetResultCodeMessage: command output (if success)
   * Config.PropertyExceptionMessage: Error message (if error)
   * Config.PropertyExceptionStack: stderr content if command failed
   * </pre>
   */
  public Properties delete(Properties p) throws Exception, RemoteException;

  /**
   * Invoke 's3cmd info' to probe an S3 bucket. The expected properties are:
   * 
   * <pre>
   * Mandatory properties:
   *  - AmazonProperties.S3_BUCKET_NAME: name of the S3 bucket to create
   * Optional properties: 
   *  - AmazonProperties.S3_LOCATION: region where to create the bucket
   * </pre>
   * 
   */
  public Properties probe(Properties p) throws Exception, RemoteException;

}
