/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Properties;

import orca.shirako.plugins.ShirakoPlugin;
import orca.shirako.plugins.config.Config;

import org.apache.log4j.Logger;

import edu.umass.cs.geni.dicloud.handlers.S3Handler;

/**
 * TODO: Def here
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class S3Server extends UnicastRemoteObject implements S3Interface
{
  private static final long  serialVersionUID      = 1985829220648345329L;
  private S3Handler          handler;
  private ShirakoPlugin      plugin;
  private Logger             logger                = Logger
	                                                   .getLogger(S3Server.class);

  public static final String RMI_NAME_REGISTRATION = "dicloud/s3";

  /**
   * @throws RemoteException
   */
  public S3Server(ShirakoPlugin plugin, S3Handler handler)
	  throws RemoteException
  {
	this.plugin = plugin;
	this.handler = handler;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.umass.cs.geni.dicloud.server.S3Interface#join(
   * java.util.Properties)
   */
  public Properties create(Properties p) throws Exception, RemoteException
  {
	handler.join(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if (!"0".equals(result))
	{
	  logger.debug("S3 Join result code: " + result);
	  logger.debug("command status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to create s3 bucket ("
		  + props.getProperty(Config.PropertyExceptionStack) + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.umass.cs.geni.dicloud.server.S3Interface#leave(
   * java.util.Properties)
   */
  public Properties delete(Properties p) throws Exception, RemoteException
  {
	handler.leave(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if (!"0".equals(result))
	{
	  logger.debug("S3 Leave result code: " + result);
	  logger.debug("Status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to delete s3 bucket ("
		  + props.getProperty(Config.PropertyExceptionStack) + ")");
	}
	return props;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.umass.cs.geni.dicloud.server.S3Interface#probe(java.lang.Object,
   * java.util.Properties)
   */
  public Properties probe(Properties p) throws Exception, RemoteException
  {
	handler.probe(null, p);
	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if (!"0".equals(result))
	{
	  logger.debug("S3 Probe result code: " + result);
	  logger.debug("Status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  logger.debug("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  logger.debug("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to probe s3 bucket ("
		  + props.getProperty(Config.PropertyExceptionStack) + ")");
	}
	return props;
  }

}
